﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

//************************************************************
/// <summary>
/// セッションデータを管理するクラス
/// </summary>
//************************************************************
public static class NlpSessionDataProvider
{
	//----------------------------------------
	//メソッド
	//----------------------------------------
	/// <summary>
	/// セッションに保存したクラスのインスタンスを取得します。
	/// セッションにクラスのインスタンスが存在しない場合には、新しいインスタンスを生成して返します。
	/// 保存するクラスはNlpSessionDataBaseを継承している必要があります。
	/// </summary>
	/// <typeparam name="T">セッションに保存するクラスの型</typeparam>
	/// <returns>セッションに保存するクラスのインスタンス</returns>
	public static T Session<T>() where T : NlpSessionDataBase, new()
	{
		T sessionDataObject = GetSession<T>();

		if (sessionDataObject == null)
			sessionDataObject = new T();

		return (T)(object)sessionDataObject;
	}
	/// <summary>
	/// 指定された型のクラスのインスタンスが、セッションに保存されているかどうかを取得します。
	/// </summary>
	/// <typeparam name="T">確認するクラスの型</typeparam>
	/// <returns>セッションに保存されているかどうか</returns>
	public static bool HasSession<T>()
	{
		return GetSession<T>() == null ? false : true;
	}
	/// <summary>
	/// すべてのセッションを破棄します。
	/// </summary>
	public static void Clear()
	{
		HttpContext.Current.Session.Clear();
	}
	/// <summary>
	/// 指定された型のクラスのインスタンスを保存するセッションを破棄します。
	/// </summary>
	/// <typeparam name="T">破棄するクラスの型</typeparam>
	public static void RemoveSession<T>()
	{
		DoRemoveSession(typeof(T).ToString());
	}
	/// <summary>
	/// セッションデータを取得します
	/// </summary>
	/// <typeparam name="T">取得するクラスの型</typeparam>
	/// <returns>指定された型のインスタンス</returns>
	private static T GetSession<T>()
	{
		return (T)(object)HttpContext.Current.Session[typeof(T).ToString()];
	}
	/// <summary>
	/// セッションデータを破棄します。
	/// </summary>
	/// <param name="typeName">破棄するセッションデータの型の名前</param>
	private static void DoRemoveSession(string typeName)
	{
		HttpContext.Current.Session.Remove(typeName);
	}
}

//************************************************************
/// <summary>
/// NlpSessionでセッションに格納するクラスの基底クラス
/// </summary>
//************************************************************
[Serializable]
public class NlpSessionDataBase
{
	//----------------------------------------
	//メソッド
	//----------------------------------------
	/// <summary>
	/// objectをセッションに保存します。
	/// </summary>
	/// <param name="sessionObject">保存するobject</param>
	protected void SetSession(object sessionObject)
	{
		string name = sessionObject.GetType().ToString();
		HttpContext.Current.Session[sessionObject.GetType().ToString()] = sessionObject;
	}
	/// <summary>
	/// valueプロパティをセッションに保存します。
	/// </summary>
	/// <typeparam name="T">セッションに保存する型</typeparam>
	/// <param name="data">セッションに保存する値を格納する変数</param>
	/// <param name="value">プロパティより受け渡された値</param>
	protected void SetValue<T>(out T data, T value)
	{
		data = value;
		this.SetSession(this);
	}
}

//************************************************************
/// <summary>
/// セッションに格納するデータを保持するクラスのサンプル
/// </summary>
//************************************************************
[Serializable]
public class SessionData : NlpSessionDataBase
{
	//----------------------------------------
	//変数宣言
	//----------------------------------------
	/// <summary>
	/// セッションに保存する値を格納する変数
	/// </summary>
	private int mKaiinID = 0;
	private String mKaiinNo = "";
	private DateTime mBirthDateTime;

	//----------------------------------------
	//メソッド
	//----------------------------------------
	/// <summary>
	/// 例：会員IDを格納するプロパティ。valueをメンバ変数に保存する場合には、SetValueメソッドを利用してください。
	/// </summary>
	public int KaiinID
	{
		get { return this.mKaiinID; }
		set { this.SetValue<int>(out this.mKaiinID, value); }
	}
	/// <summary>
	/// 例：会員番号を格納するプロパティ。valueをメンバ変数に保存する場合には、SetValueメソッドを利用してください。
	/// </summary>
	public string KaiinNo
	{
		get { return this.mKaiinNo; }
		set { this.SetValue<string>(out this.mKaiinNo, value); }
	}
	/// <summary>
	/// 例：誕生日を格納するプロパティ。valueをメンバ変数に保存する場合には、SetValueメソッドを利用してください。
	/// </summary>
	public DateTime BirthDateTime
	{
		get { return this.mBirthDateTime; }
		set { this.SetValue<DateTime>(out this.mBirthDateTime, value); }
	}
}

